/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.robokind.demo.robot.replication.lifecycle;

import org.jflux.api.core.Source;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import org.jflux.api.core.config.Configuration;
import org.jflux.avrogen.EventListRecord;
import org.osgi.framework.BundleContext;
import org.robokind.api.common.lifecycle.utils.ManagedServiceGroup;
import org.robokind.api.common.osgi.OSGiUtils;
import static org.robokind.impl.messaging.config.RKMessagingConfigUtils.*;
import static org.robokind.impl.messaging.utils.ConnectionManager.*;
import static org.robokind.impl.messaging.config.MessagingLifecycleGroupConfigUtils.*;
import static org.robokind.api.common.lifecycle.config.RKManagedGroupConfigUtils.*;

/**
 *
 * @author Matthew Stevenson
 */
public class ReplicationSenderGroupLifecycleUtils {
    public final static String EVENT_LIST_RECORD_ENCODE_CONFIG_FULL_ID = EventListRecord.class.toString();
    public final static String REPLICATION_DESTINATION_ID = "replicationDestinationId";
    public final static String REPLICATION_CONNECTION_ID = "replicationConnectionId";
    public final static String REPLICATION_SESSION_ID = "replicationSessionId";
    public final static String MESSAGE_PACKER_ID = "messagePackerId";
    public final static String MESSAGE_HEADER_ID = "messageHeaderId";
    public final static String REMOTE_REPLICATION_NODE_ID = ReplicationSenderLifecycleUtils.PROP_REPLICATION_SENDER_NODE_ID;
    public final static String REPLICATION_SENDER_ID = "replicationSenderId";
    
    public final static String BUF_EVENT_FACTORY_ID = "bufEventFactoryId";
    public final static String BUF_EVENT_ENCODER_ID = "bufEventEncodeId";
    
    public final static String EVENT_COLLECTOR_ID = "eventCollectorId";
    public final static String EVENT_BUFFER_TRACKER_ID = "eventBufferTrackerId";
    public final static String LINK_SENDER_ID = "senderLinkId";
    public final static String LINK_COLLECTION_TIMER_ID = "collectionTimerLinkId";
    public final static String COLLECTION_TIMER_ID = "collectionTimerId";
    
    public final static String GROUP_TYPE = "ReplicationSenderGroup";
    private final static String theIdFormat = "%s/" + GROUP_TYPE + "/%s";
    
    public static String id(String groupId, String part){
        return String.format(theIdFormat, groupId, part);
    }
    
    public static List<Configuration<String>> 
            getReplicationSenderLifecycleConfigs(String groupId){
        return ReplicationSenderLifecycleUtils.getLifecycleConfigs(
                SERIALIZATION_CONFIG, EVENT_LIST_RECORD_ENCODE_CONFIG_FULL_ID, 
                PROP_DESTINATION_ID, id(groupId, REPLICATION_DESTINATION_ID), 
                PROP_CONNECTION_ID, id(groupId, REPLICATION_SESSION_ID), 
                id(groupId, MESSAGE_PACKER_ID), 
                id(groupId, MESSAGE_HEADER_ID), 
                id(groupId, REMOTE_REPLICATION_NODE_ID), 
                id(groupId, REPLICATION_SENDER_ID));
    }
    
    public static List<Configuration<String>> getJMSLifecycleConfigs(
            String groupId, String destinationConfId, String connectionConfId){
        List<Configuration<String>> l = new ArrayList();
        l.add(makeSelfBuildingLifecycle(buildDestinationLifecycleConfig(
                destinationConfId,id(groupId, REPLICATION_DESTINATION_ID))));
        l.add(makeSelfBuildingLifecycle(buildConnectionLifecycleConfig(
                connectionConfId, id(groupId, REPLICATION_CONNECTION_ID))));
        l.add(makeSelfBuildingLifecycle(buildSessionLifecycleConfig(
                id(groupId, REPLICATION_CONNECTION_ID), 
                id(groupId, REPLICATION_SESSION_ID))));
        return l;
    }
    
    public static List<Configuration<String>> getReplicationBufferLifecycleConfigs(
            String groupId, String eventBufferId, String encodeConfigId, 
            Properties bufferProps, int bufferCapacity){
        return EventBufferUtils.getLifecycleConfigs(
                SERIALIZATION_CONFIG, 
                encodeConfigId, 
                id(groupId, eventBufferId + "/" + BUF_EVENT_FACTORY_ID), 
                eventBufferId,
                groupId, 
                id(groupId, eventBufferId + "/" + BUF_EVENT_ENCODER_ID), 
                eventBufferId, bufferProps, bufferCapacity);
    }
    
    public static List<Configuration<String>> getAlternateReplicationBufferLifecycleConfigs(
            String groupId, String eventBufferId, String encodeConfigId, 
            Source valueSource){
        return EventBufferUtils.getAlternateLifecycleConfigs(
                SERIALIZATION_CONFIG, 
                encodeConfigId, 
                id(groupId, eventBufferId + "/" + BUF_EVENT_FACTORY_ID), 
                eventBufferId,
                groupId, 
                id(groupId, eventBufferId + "/" + BUF_EVENT_ENCODER_ID), 
                eventBufferId, valueSource);
    }
    
    public static List<Configuration<String>> getCollectorLifecycleConfigs(
            String groupId, BundleContext context, Properties collectorProps, 
            long initialDelay, long period, Properties timerProps){
        return CollectorLifecycleUtils.buildTimedCollector( context, 
                EVENT_COLLECTOR_ID, id(groupId,EVENT_COLLECTOR_ID), 
                collectorProps, 
                EVENT_BUFFER_TRACKER_ID, id(groupId,EVENT_BUFFER_TRACKER_ID), 
                OSGiUtils.createFilter(ManagedServiceGroup.PROP_GROUP_ID, groupId), 
                LINK_SENDER_ID, 
                REMOTE_REPLICATION_NODE_ID, id(groupId, REMOTE_REPLICATION_NODE_ID), 
                LINK_COLLECTION_TIMER_ID, 
                COLLECTION_TIMER_ID, id(groupId, COLLECTION_TIMER_ID), 
                initialDelay, period, timerProps);
    }
}
